@@ -11,21 +11,19 @@ from matplotlib import rcParams
 from matplotlib.colors import ListedColormap
 from matplotlib.gridspec import GridSpec
 import matplotlib
+import seaborn as sns
 
-from ..common.utils import combine_like_features, is_outlier
+from ..common.utils import is_outlier
+from ..common.contrib_utils import combine_like_features
 import shap
 
-
 class PlotStructure:
     """
     Plot handles figure and subplot generation
     """
 
-    def __init__(self, BASE_FONT_SIZE = 12):
-    
-        # Setting the font style to serif
-        rcParams["font.family"] = "serif"
-
+    def __init__(self, BASE_FONT_SIZE=12, set_seaborn=True):
+        
         GENERIC_FONT_SIZE_NAMES = [
             "teensie",
             "tiny",
@@ -34,47 +32,58 @@ class PlotStructure:
             "big",
             "large",
             "huge",
-        ]
+            ]
 
         FONT_SIZES_ARRAY = np.arange(-6, 8, 2) + BASE_FONT_SIZE
 
         self.FONT_SIZES = {
-            name: size for name, size in zip(GENERIC_FONT_SIZE_NAMES, FONT_SIZES_ARRAY)
-        }
+                name: size for name, size in zip(GENERIC_FONT_SIZE_NAMES, FONT_SIZES_ARRAY)
+            }
+        
+        if set_seaborn:
+            sns.set_theme()
+        
+        # Setting the font style to serif
+        rcParams["font.family"] = "serif"
 
         plt.rc("font", size=self.FONT_SIZES["normal"])  # controls default text sizes
         plt.rc("axes", titlesize=self.FONT_SIZES["tiny"])  # fontsize of the axes title
-        plt.rc("axes", labelsize=self.FONT_SIZES["normal"])  # fontsize of the x and y labels
         plt.rc(
-        "xtick", labelsize=self.FONT_SIZES["teensie"]
-        )  # fontsize of the x-axis tick marks
+                 "axes", labelsize=self.FONT_SIZES["normal"]
+                )  # fontsize of the x and y labels
         plt.rc(
-        "ytick", labelsize=self.FONT_SIZES["teensie"]
-        )  # fontsize of the y-axis tick marks
+                "xtick", labelsize=self.FONT_SIZES["teensie"]
+                )  # fontsize of the x-axis tick marks
+        plt.rc(
+                "ytick", labelsize=self.FONT_SIZES["teensie"]
+                )  # fontsize of the y-axis tick marks
         plt.rc("legend", fontsize=self.FONT_SIZES["teensie"])  # legend fontsize
-        plt.rc("figure", titlesize=self.FONT_SIZES["big"])  # fontsize of the figure title
+        plt.rc(
+                "figure", titlesize=self.FONT_SIZES["big"]
+                )  # fontsize of the figure title
 
     def get_fig_props(self, n_panels, **kwargs):
         """Determine appropriate figure properties"""
         width_slope = 0.875
         height_slope = 0.45
-        intercept = (3.0 - width_slope)
-        figsize = (min((n_panels*width_slope)+intercept,19) , 
-                   min((n_panels*height_slope)+intercept,12)
-                  )
-        
-        wspace=(-0.03*n_panels)+0.85
-        hspace=(0.0175*n_panels)+0.3
-        
-        n_columns = kwargs.get('n_columns', 3)
-        wspace = wspace+0.25 if n_columns > 3 else wspace
-        
+        intercept = 3.0 - width_slope
+        figsize = (
+            min((n_panels * width_slope) + intercept, 19),
+            min((n_panels * height_slope) + intercept, 12),
+        )
+
+        wspace = (-0.03 * n_panels) + 0.85
+        hspace = (0.0175 * n_panels) + 0.3
+
+        n_columns = kwargs.get("n_columns", 3)
+        wspace = wspace + 0.25 if n_columns > 3 else wspace
+
         kwargs["figsize"] = kwargs.get("figsize", figsize)
         kwargs["wspace"] = kwargs.get("wspace", wspace)
         kwargs["hspace"] = kwargs.get("hspace", hspace)
-        
-        return kwargs    
-        
+
+        return kwargs
+
     def create_subplots(self, n_panels, **kwargs):
         """
         Create a series of subplots (MxN) based on the
@@ -121,6 +130,9 @@ class PlotStructure:
             figsize=figsize,
             dpi=300,
         )
+
+        fig.patch.set_facecolor("white")
+
         plt.subplots_adjust(wspace=wspace, hspace=hspace)
 
         if delete:
@@ -142,7 +154,7 @@ class PlotStructure:
         n_columns = kwargs.get("n_columns", 3)
 
         fig = plt.figure(figsize=figsize, dpi=300)
-
+        fig.patch.set_facecolor("white")
         extra_row = 0 if (n_panels % n_columns) == 0 else 1
 
         nrows = ratio * (int(n_panels / n_columns) + extra_row)
@@ -213,7 +225,7 @@ class PlotStructure:
         return fig, main_axes, top_axes, rhs_axes, n_rows
 
     def axes_to_iterator(self, n_panels, axes):
-        """Turns axes list into iterable """
+        """Turns axes list into iterable"""
         if isinstance(axes, list):
             return axes
         else:
@@ -221,7 +233,13 @@ class PlotStructure:
             return ax_iterator
 
     def set_major_axis_labels(
-        self, fig, xlabel=None, ylabel_left=None, ylabel_right=None, title=None, **kwargs
+        self,
+        fig,
+        xlabel=None,
+        ylabel_left=None,
+        ylabel_right=None,
+        title=None,
+        **kwargs,
     ):
         """
         Generate a single X- and Y-axis labels for
@@ -229,6 +247,7 @@ class PlotStructure:
         """
         fontsize = kwargs.get("fontsize", self.FONT_SIZES["normal"])
         labelpad = kwargs.get("labelpad", 15)
+        ylabel_right_color = kwargs.get("ylabel_right_color", "k")
 
         # add a big axis, hide frame
         ax = fig.add_subplot(111, frameon=False)
@@ -249,10 +268,18 @@ class PlotStructure:
             )
 
             ax_right.yaxis.set_label_position("right")
-            ax_right.set_ylabel(ylabel_right, labelpad=2 * labelpad, fontsize=fontsize)
+            ax_right.set_ylabel(
+                ylabel_right,
+                labelpad=2 * labelpad,
+                fontsize=fontsize,
+                color=ylabel_right_color,
+            )
+            ax_right.grid(False)
+
+        ax.set_title(title)
+        
+        ax.grid(False)
 
-        ax.set_title(title)     
-            
         return ax
 
     def set_row_labels(self, labels, axes, pos=-1, pad=1.15, rotation=270, **kwargs):
@@ -282,14 +309,14 @@ class PlotStructure:
                 alpha=0.65,
             )
 
-    def add_alphabet_label(self, n_panels, axes, pos=(0.9, 0.09), **kwargs):
+    def add_alphabet_label(self, n_panels, axes, pos=(0.9, 0.09), alphabet_fontsize=10, **kwargs):
         """
         A alphabet character to each subpanel.
         """
-        fontsize = kwargs.get("fontsize", 10)
-        alphabet_list = [chr(x) for x in range(ord("a"), ord("z") + 1)] + \
-            [f"{chr(x)}{chr(x)}" for x in range(ord("a"), ord("z") + 1)]
-       
+        alphabet_list = [chr(x) for x in range(ord("a"), ord("z") + 1)] + [
+            f"{chr(x)}{chr(x)}" for x in range(ord("a"), ord("z") + 1)
+        ]
+
         ax_iterator = self.axes_to_iterator(n_panels, axes)
 
         for i, ax in enumerate(ax_iterator):
@@ -297,7 +324,7 @@ class PlotStructure:
                 pos[0],
                 pos[1],
                 f"({alphabet_list[i]})",
-                fontsize=fontsize,
+                fontsize=alphabet_fontsize,
                 alpha=0.8,
                 ha="center",
                 va="center",
@@ -308,21 +335,27 @@ class PlotStructure:
         """
         Convert decimals (less 0.01) to 10^e notation
         """
-        #f = mticker.ScalarFormatter(useOffset=False, useMathText=True)
-        #g = lambda x, pos: "${}$".format(f._formatSciNotation("%1.10e" % x))
+        # f = mticker.ScalarFormatter(useOffset=False, useMathText=True)
+        # g = lambda x, pos: "${}$".format(f._formatSciNotation("%1.10e" % x))
 
         if colorbar and np.absolute(np.amax(ydata)) <= 0.01:
-            #colorbar.ax.yaxis.set_major_formatter(mticker.FuncFormatter(g))
-            colorbar.ax.ticklabel_format(style='sci', )
+            # colorbar.ax.yaxis.set_major_formatter(mticker.FuncFormatter(g))
+            colorbar.ax.ticklabel_format(
+                style="sci",
+            )
             colorbar.ax.tick_params(axis="y", labelsize=5)
         elif ax:
             if np.absolute(np.amax(xdata)) <= 0.01:
-                ax.ticklabel_format(style='sci', )
-                #ax.xaxis.set_major_formatter(mticker.FuncFormatter(g))
+                ax.ticklabel_format(
+                    style="sci",
+                )
+                # ax.xaxis.set_major_formatter(mticker.FuncFormatter(g))
                 ax.tick_params(axis="x", labelsize=5, rotation=45)
             if np.absolute(np.amax(ydata)) <= 0.01:
-                #ax.yaxis.set_major_formatter(mticker.FuncFormatter(g))
-                ax.ticklabel_format(style='sci', )
+                # ax.yaxis.set_major_formatter(mticker.FuncFormatter(g))
+                ax.ticklabel_format(
+                    style="sci",
+                )
                 ax.tick_params(axis="y", labelsize=5, rotation=45)
 
     def calculate_ticks(
@@ -331,18 +364,26 @@ class PlotStructure:
         ax=None,
         upperbound=None,
         lowerbound=None,
-        round_to=1,
+        round_to=5,
         center=False,
     ):
         """
         Calculate the y-axis ticks marks for the line plots
         """
         if ax is not None:
-            upperbound = round(ax.get_ybound()[1], round_to)
-            lowerbound = round(ax.get_ybound()[0], round_to)
+            upperbound = round(ax.get_ybound()[1], 5)
+            lowerbound = round(ax.get_ybound()[0], 5)
 
         max_value = max(abs(upperbound), abs(lowerbound))
-        if max_value > 10:
+        if 0 < max_value < 1:
+            if max_value < 0.1:
+                round_to = 3
+            else:
+                round_to = 5
+
+        elif 5 < max_value < 10:
+            round_to = 2
+        else:
             round_to = 0
 
         def round_to_a_base(a_number, base=5):
@@ -356,9 +397,11 @@ class PlotStructure:
             values = np.round(values, round_to)
         else:
             dy = upperbound - lowerbound
-            fit = np.floor(dy / (nticks - 1)) + 1
-            dy_new = (nticks - 1) * fit
-            values = np.linspace(lowerbound, lowerbound + dy_new, nticks)
+            # deprecated 8 March 2022 by Monte.
+            if round_to > 2:
+                fit = np.floor(dy / (nticks - 1)) + 1
+                dy = (nticks - 1) * fit
+            values = np.linspace(lowerbound, lowerbound + dy, nticks)
             values = np.round(values, round_to)
 
         return values
@@ -419,15 +462,20 @@ class PlotStructure:
         Set a single legend on the bottom of a figure
         for a set of subplots.
         """
-        handles, labels = ax.get_legend_handles_labels()
+        fontsize = kwargs.get("fontsize", "medium")
+        ncol = kwargs.get("ncol", 3)
+        handles = kwargs.get("handles", None)
+        labels = kwargs.get("labels", None)
+        if handles is None and labels is None:
+            handles, labels = ax.get_legend_handles_labels()
 
         if n_panels > 3:
             bbox_to_anchor = (0.5, -0.35)
         else:
             bbox_to_anchor = (0.5, -0.5)
 
-        bbox_to_anchor = kwargs.get('bbox_to_anchor', bbox_to_anchor)    
-            
+        bbox_to_anchor = kwargs.get("bbox_to_anchor", bbox_to_anchor)
+
         # Shrink current axis's height by 10% on the bottom
         box = major_ax.get_position()
         major_ax.set_position(
@@ -442,7 +490,8 @@ class PlotStructure:
             bbox_to_anchor=bbox_to_anchor,
             fancybox=True,
             shadow=True,
-            ncol=3,
+            ncol=ncol,
+            fontsize=fontsize,
         )
 
     def set_minor_ticks(self, ax):
@@ -450,20 +499,39 @@ class PlotStructure:
         Adds minor tick marks to the x- and y-axis to a subplot ax
         to increase readability.
         """
-        ax.xaxis.set_minor_locator(AutoMinorLocator())
-        ax.yaxis.set_minor_locator(AutoMinorLocator())
+        ax.xaxis.set_minor_locator(AutoMinorLocator(n=3))
+        ax.yaxis.set_minor_locator(AutoMinorLocator(n=3))
 
-    def set_n_ticks(self, ax, option="y"):
+    def set_n_ticks(self, ax, option="y", nticks=5):
         """
         Set the max number of ticks per x- and y-axis for a
         subplot ax
         """
         if option == "y" or option == "both":
-            ax.yaxis.set_major_locator(MaxNLocator(5))
-            ax.yaxis.set_major_locator(MaxNLocator(4))
+            ax.yaxis.set_major_locator(MaxNLocator(nticks))
+            
         if option == "x" or option == "both":
-            ax.xaxis.set_major_locator(MaxNLocator(5))
-            ax.xaxis.set_major_locator(MaxNLocator(4))
+            ax.xaxis.set_major_locator(MaxNLocator(nticks))
+
+    def make_twin_ax(self, ax):
+        """
+        Create a twin axis on an existing axis with a shared x-axis
+        """
+        # align the twinx axis
+        twin_ax = ax.twinx()
+
+        # Turn twin_ax grid off.
+        twin_ax.grid(False)
+
+        # Set ax's patch invisible
+        ax.patch.set_visible(False)
+        # Set axtwin's patch visible and colorize it in grey
+        twin_ax.patch.set_visible(True)
+
+        # move ax in front
+        ax.set_zorder(twin_ax.get_zorder() + 1)
+
+        return twin_ax
 
     def despine_plt(self, ax):
         """
@@ -474,13 +542,44 @@ class PlotStructure:
         ax.spines["left"].set_visible(False)
         ax.spines["bottom"].set_visible(False)
 
+    def annotate_bars(self, ax, num1, num2, y, width, dh=0.01, barh=0.05, delta=0):
+        """ 
+        Annotate barplot with connections between correlated variables. 
+
+        Parameters
+        ----------------
+            num1: index of left bar to put bracket over
+            num2: index of right bar to put bracket over
+            y: centers of all bars (like plt.barh() input)
+            width: widths of all bars (like plt.barh() input)
+            dh: height offset over bar / bar + yerr in axes coordinates (0 to 1)
+            barh: bar height in axes coordinates (0 to 1)
+            delta : shifting of the annotation when multiple annotations would overlap
+        """
+        lx, ly = y[num1], width[num1]
+        rx, ry = y[num2], width[num2]
+
+        ax_y0, ax_y1 = plt.gca().get_xlim()
+        dh *= (ax_y1 - ax_y0)
+        barh *= (ax_y1 - ax_y0)
+
+        y = max(ly, ry) + dh
+
+        barx = [lx, lx, rx, rx]
+        bary = [y, y+barh, y+barh, y]
+        mid = ((lx+rx)/2, y+barh)
+
+        ax.plot(np.array(bary)+delta, barx, alpha=0.8, clip_on=False)   
+        
+    '''
+    Deprecated 14 March 2022. 
     def annotate_bars(self, ax, bottom_idx, top_idx, x=0, **kwargs):
         """
         Adds a square bracket that contains two points. Used to
         connect predictors in the predictor ranking plot
         for highly correlated pairs.
         """
-        color = kwargs.get('color', "xkcd:slate gray")
+        color = kwargs.get("color", "xkcd:slate gray")
         ax.annotate(
             "",
             xy=(x, bottom_idx),
@@ -494,22 +593,27 @@ class PlotStructure:
                 linewidth=0.5,
             ),
         )
+    '''
 
     def get_custom_colormap(self, vals, **kwargs):
         """Get a custom colormap"""
-        cmap = kwargs.get('cmap', matplotlib.cm.PuOr)
-        bounds = np.linspace(np.nanpercentile(vals, 0),
-                     np.nanpercentile(vals, 100),
-                     10)
+        cmap = kwargs.get("cmap", matplotlib.cm.PuOr)
+        bounds = np.linspace(np.nanpercentile(vals, 0), np.nanpercentile(vals, 100), 10)
+
+        norm = matplotlib.colors.BoundaryNorm(
+            bounds,
+            cmap.N,
+        )
+        mappable = matplotlib.cm.ScalarMappable(
+            norm=norm,
+            cmap=cmap,
+        )
 
-        norm = matplotlib.colors.BoundaryNorm(bounds, cmap.N, )
-        mappable = matplotlib.cm.ScalarMappable(norm=norm, cmap=cmap, )
-        
         return mappable, bounds
-        
-    def add_ice_colorbar(self, fig, ax,  mappable, cb_label, cdata, fontsize, **kwargs):
-        """Add a colorbar to the right of a panel to 
-            accompany ICE color-coded plots"""
+
+    def add_ice_colorbar(self, fig, ax, mappable, cb_label, cdata, fontsize, **kwargs):
+        """Add a colorbar to the right of a panel to
+        accompany ICE color-coded plots"""
         cb = plt.colorbar(mappable, ax=ax, pad=0.2)
         cb.set_label(cb_label, size=fontsize)
         cb.ax.tick_params(labelsize=fontsize)
@@ -518,7 +622,7 @@ class PlotStructure:
         bbox = cb.ax.get_window_extent().transformed(fig.dpi_scale_trans.inverted())
         cb.ax.set_aspect((bbox.height - 0.7) * 20)
         self._to_sci_notation(ax=None, colorbar=cb, ydata=cdata)
-        
+
     def add_colorbar(
         self,
         fig,
@@ -529,7 +633,7 @@ class PlotStructure:
         cax=None,
         **kwargs,
     ):
-        """ Adds a colorbar to the right of a panel"""
+        """Adds a colorbar to the right of a panel"""
         # Add a colobar
         orientation = kwargs.get("orientation", "vertical")
         pad = kwargs.get("pad", 0.1)
@@ -563,5 +667,5 @@ class PlotStructure:
         # cbar.ax.set_aspect((bbox.height - 0.7) * 20)
 
     def save_figure(self, fname, fig=None, bbox_inches="tight", dpi=300, aformat="png"):
-        """ Saves the current figure """
+        """Saves the current figure"""
         plt.savefig(fname=fname, bbox_inches=bbox_inches, dpi=dpi, format=aformat)
